#!/bin/bash

# Function to display commands
exe() { echo "-- cmd# ${@/eval/}" ; "$@" ; }

#set -x     # activate command traces
#trap 'echo "# $BASH_COMMAND"' DEBUG

function pause(){
#read -s -n 1 -p "Press any key to continue . . ."
echo ""
}


USAGE="
Usage:
    
vcpe_start <vcpe_name> <vnf_tunnel_ip> <home_tunnel_ip> <vcpe_private_ip> <vcpe_public_ip> <dhcpd_conf_file>
    being:
        <vcpe_name>: the name of the network service instance in OSM 
        <vnf_tunnel_ip>: the ip address for the vnf side of the tunnel
        <home_tunnel_ip>: the ip address for the home side of the tunnel
        <vcpe_private_ip>: the private ip address for the vcpe
        <vcpe_public_ip>: the public ip address for the vcpe (10.2.2.0/24)
"

if [[ $# -ne 6 ]]; then
        echo ""       
    echo "ERROR: incorrect number of parameters"
    echo "$USAGE"
    exit 1
fi

VNF1="mn.dc1_$1-1-vcpe-ha-1"
VNFTUNIP="$2"
HOMETUNIP="$3"
VCPEPRIVIP="$4"
VCPEPUBIP="$5"
VCPEHOMEIP="$6"

echo "--"
echo "-- Creating vCPE NS..."
#sudo osm ns-create
exe eval osm ns-create --nsd_name vCPE --ns_name vcpe-1 --vim_account emu-vim

while [ "$( docker container inspect -f '{{.State.Running}}' $VNF1 2> /dev/null )" != 'true' ]; do 
    echo "--   OSM $VNF1 docker not ready, waiting 5s..."; 
    sleep 5;
done
echo "-- OSM $VNF1 docker ready!"

ETH11=`sudo docker exec -it $VNF1 ifconfig | grep eth1 | awk '{print $1}'`
prov=`sudo docker exec -it $VNF1 ifconfig -a | awk '/192\.168\./ && /inet/{print $2}'`
IP11="${prov:5:14}"
echo "-- ETH11=$ETH11, IP11=$IP11"

##################### VNFs Settings #####################
## 0. Iniciar el Servicio OpenVirtualSwitch en cada VNF:
echo "--"
echo "-- OVS Starting..."
exe eval sudo docker exec -it $VNF1 /usr/share/openvswitch/scripts/ovs-ctl start


echo "--"
echo "-- Connecting vCPE service with AccessNet and ExtNet..."
exe eval sudo ovs-docker add-port AccessNet veth0 $VNF1
#sleep 5
exe eval sudo ovs-docker add-port ExtNet veth1 $VNF1
#sleep 5

pause

echo "--"
echo "-- Start home and server virtual scenarios..."
echo "--"
sudo vnx -f nfv3_home_lxc_ubuntu64.xml -t
sudo vnx -f nfv3_server_lxc_ubuntu64.xml -t

# Wait till brg1 is ready 
while ! sudo lxc-attach brg1 -- bash -c "ovs-vsctl show | grep br0" 2> /dev/null; do 
    echo "--   waiting for brg1 to start..."; sleep 3; 
done
    echo "-- brg1 ready"; sleep 3; 

pause

echo "--"
echo "-- Setting vCPE VNF..."
echo "--"
echo "-- Creating bridge br1..."
exe eval sudo docker exec -it $VNF1 ovs-vsctl add-br br1

echo "-- Config vcpe IP addresses..."
exe eval sudo docker exec -it $VNF1 ifconfig veth0 $VNFTUNIP netmask 255.255.255.0
exe eval sudo docker exec -it $VNF1 ifconfig veth1 $VCPEPUBIP netmask 255.255.255.0
exe eval sudo docker exec -it $VNF1 ifconfig br1 $VCPEHOMEIP netmask 255.255.255.0
exe eval sudo docker exec -it $VNF1 ifconfig br1 mtu 1350

echo "-- Creating VXLAN tunnel between br0 (brg1) and br1 (vcpe)..."
exe eval sudo docker exec -it $VNF1 ovs-vsctl add-port br1 vxlan0 -- set interface vxlan0 type=vxlan options:remote_ip=$HOMETUNIP 
# Los tuneles creados desde linux parecen no funcionar en Alpine Linux
#exe eval sudo docker exec -it $VNF1 ip link add vxlan0 type vxlan id 0 remote $HOMETUNIP dstport 4789
#exe eval sudo docker exec -it $VNF1 ovs-vsctl add-port br1 vxlan0
#exe eval sudo docker exec -it $VNF1 ifconfig vxlan0 up

pause

## VCPE: asignar rutas a la interfaz de salida y a la interfaz de entrada.

echo "-- Set vcpe default route..."

exe eval sudo docker exec -it $VNF1 ip route del 0.0.0.0/0 via 172.17.0.1
exe eval sudo docker exec -it $VNF1 ip route add 0.0.0.0/0 via 10.2.3.254

pause

## En VNF:vcpe y home configuramos NAT para dar salida a Internet y redirigir conexiones entrantes al router
######en el puerto 8213 al docker de Home Assistant. 
exe eval docker cp /usr/bin/vnx_config_nat  $VNF1:/usr/bin
exe eval sudo docker exec -it $VNF1 /usr/bin/vnx_config_nat br1 veth1

pause

exe eval sudo docker exec -it $VNF1 iptables -t nat -A PREROUTING -p tcp -d 10.2.3.1 --dport 8123 -j DNAT --to-destination $IP11:8123
exe eval sudo docker exec -it $VNF1 iptables -t nat -A POSTROUTING ! -s 127.0.0.1 -j MASQUERADE
exe eval sudo docker exec -it $VNF1 "iptables-save > /etc/iptables.rules"
exe eval sudo docker exec -it $VNF1 "iptables-restore < /etc/iptables.rules"

exe eval sudo lxc-attach r1 -- dhclient
IPEXTR1=$( sudo lxc-attach r1 -- ifconfig eth9 | grep "inet " | awk '{ print $2 }' )
exe eval sudo ip route add 10.2.3.0/24 via $IPEXTR1

pause

#------------------------------------------------------------------------------------------------------------#


## 7. Instalamos mosquitto en los equipos de la red residencial.
#sleep 3

#sudo docker exec -it $VNF1 apk add openssh
#sudo docker exec -it $VNF1 apk add sshpass
#sudo docker exec -it $VNF1 apk add openrc
#sudo docker exec -it $VNF1 rc-update add sshd
#sudo docker exec -it $VNF1 rc-status
#sudo docker exec -it $VNF1 touch /run/openrc/softlevel
#sudo docker exec -it $VNF1 /etc/init.d/sshd start
#sudo docker exec -it $VNF1 sed -i 's/#PermitRootLogin prohibit-password/PermitRootLogin yes/' /etc/ssh/sshd_config
#sudo docker exec -it $VNF1 sed '$ a StrictHostKeyChecking no' /etc/ssh/sshd_config 
#sudo docker exec -it $VNF1 mkdir -p ~/.ssh
#sudo docker exec -it $VNF1 ssh-keyscan 10.255.0.2 >> ~/.ssh/known_hosts
#sudo docker exec -it $VNF1 /etc/init.d/sshd restart

#sleep 15

pause

#sudo lxc-attach -n br1 -- apt-get update
#sudo lxc-attach -n br1 -- apt-get -y install sshpass
#sleep 35
#sudo lxc-attach -n br1 -- sed -i 's/#PermitRootLogin prohibit-password/PermitRootLogin yes/' /etc/ssh/sshd_config
#sudo lxc-attach -n br1 -- sed -i 's/#PubkeyAuthentication yes/PubkeyAuthentication no/' /etc/ssh/sshd_config
#sudo lxc-attach -n br1 -- ssh-keyscan 10.255.0.1 >> ~/.ssh/known_hosts
#sudo lxc-attach -n br1 --  /etc/init.d/ssh restart
#
#pause

#sudo lxc-attach -n h11 -- apt-get update 
#sudo lxc-attach -n h11 -- apt-get -y install mosquitto mosquitto-clients python3-pip
#sudo lxc-attach -n h11 -- pip3 install paho-mqtt python-etcd

exe eval docker cp conf/configuration.yaml  $VNF1:/config
#
#sudo lxc-attach -n br1 -- sudo /usr/bin/sshpass -p 'root' scp  -o StrictHostKeyChecking=no root@10.255.0.1:/config/configuration.yaml /root/
#sudo lxc-attach -n h12 -- apt-get update 
#sudo lxc-attach -n h12 -- apt-get -y install mosquitto mosquitto-clients
#sudo lxc-attach -n br1 -- apt-get update 
#sudo lxc-attach -n br1 -- apt-get -y install mosquitto mosquitto-clients
#sudo lxc-attach -n lamp1 -- apt-get update 
#sudo lxc-attach -n lamp1 -- apt-get -y install mosquitto mosquitto-clients

#------------------------------------------------------------------------------------------------------------#

## 8. Lanzamos script de Connectivity Testing.
#sleep 25
#sudo lxc-attach -n br1 -- sudo /bin/bash /root/script.sh


sudo vnx -f nfv3_home_lxc_ubuntu64.xml -x install_mosquitto 
